These functions are variants of map() that iterate over multiple arguments
simultaneously. They are parallel in the sense that each input is processed
in parallel with the others, not in the sense of multicore computing. They
share the same notion of "parallel" as base::pmax() and base::pmin().
map2() and walk2() are specialised for the two argument case; pmap()
and pwalk() allow you to provide any number of arguments in a list. Note
that a data frame is a very important special case, in which case pmap()
and pwalk() apply the function .f to each row. map_dfr(), pmap_dfr()
and map2_dfc(), pmap_dfc() return data frames created by row-binding
and column-binding respectively. They require dplyr to be installed.
Usage
map2(.x, .y, .f, ...)
map2_lgl(.x, .y, .f, ...)
map2_int(.x, .y, .f, ...)
map2_dbl(.x, .y, .f, ...)
map2_chr(.x, .y, .f, ...)
map2_raw(.x, .y, .f, ...)
map2_dfr(.x, .y, .f, ..., .id = NULL)
map2_dfc(.x, .y, .f, ...)
walk2(.x, .y, .f, ...)
pmap(.l, .f, ...)
pmap_lgl(.l, .f, ...)
pmap_int(.l, .f, ...)
pmap_dbl(.l, .f, ...)
pmap_chr(.l, .f, ...)
pmap_raw(.l, .f, ...)
pmap_dfr(.l, .f, ..., .id = NULL)
pmap_dfc(.l, .f, ...)
pwalk(.l, .f, ...)Arguments
- .x, .y
Vectors of the same length. A vector of length 1 will be recycled.
- .f
A function, formula, or vector (not necessarily atomic).
If a function, it is used as is.
If a formula, e.g.
~ .x + 2, it is converted to a function. There are three ways to refer to the arguments:For a single argument function, use
.For a two argument function, use
.xand.yFor more arguments, use
..1,..2,..3etc
This syntax allows you to create very compact anonymous functions.
If character vector, numeric vector, or list, it is converted to an extractor function. Character vectors index by name and numeric vectors index by position; use a list to index by position and name at different levels. If a component is not present, the value of
.defaultwill be returned.- ...
Additional arguments passed on to the mapped function.
- .id
Either a string or
NULL. If a string, the output will contain a variable with that name, storing either the name (if.xis named) or the index (if.xis unnamed) of the input. IfNULL, the default, no variable will be created.Only applies to
_dfrvariant.- .l
A list of vectors, such as a data frame. The length of
.ldetermines the number of arguments that.fwill be called with. List names will be used if present.
Value
An atomic vector, list, or data frame, depending on the suffix.
Atomic vectors and lists will be named if .x or the first
element of .l is named.
If all input is length 0, the output will be length 0. If any
input is length 1, it will be recycled to the length of the longest.
Details
Note that arguments to be vectorised over come before .f,
and arguments that are supplied to every call come after .f.
Examples
x <- list(1, 1, 1)
y <- list(10, 20, 30)
z <- list(100, 200, 300)
map2(x, y, ~ .x + .y)
#> [[1]]
#> [1] 11
#>
#> [[2]]
#> [1] 21
#>
#> [[3]]
#> [1] 31
#>
# Or just
map2(x, y, `+`)
#> [[1]]
#> [1] 11
#>
#> [[2]]
#> [1] 21
#>
#> [[3]]
#> [1] 31
#>
pmap(list(x, y, z), sum)
#> [[1]]
#> [1] 111
#>
#> [[2]]
#> [1] 221
#>
#> [[3]]
#> [1] 331
#>
# Matching arguments by position
pmap(list(x, y, z), function(first, second, third) (first + third) * second)
#> [[1]]
#> [1] 1010
#>
#> [[2]]
#> [1] 4020
#>
#> [[3]]
#> [1] 9030
#>
# Matching arguments by name
l <- list(a = x, b = y, c = z)
pmap(l, function(c, b, a) (a + c) * b)
#> [[1]]
#> [1] 1010
#>
#> [[2]]
#> [1] 4020
#>
#> [[3]]
#> [1] 9030
#>
# Split into pieces, fit model to each piece, then predict
by_cyl <- mtcars %>% split(.$cyl)
mods <- by_cyl %>% map(~ lm(mpg ~ wt, data = .))
map2(mods, by_cyl, predict)
#> $`4`
#> Datsun 710 Merc 240D Merc 230 Fiat 128
#> 26.47010 21.55719 21.78307 27.14774
#> Honda Civic Toyota Corolla Toyota Corona Fiat X1-9
#> 30.45125 29.20890 25.65128 28.64420
#> Porsche 914-2 Lotus Europa Volvo 142E
#> 27.48656 31.02725 23.87247
#>
#> $`6`
#> Mazda RX4 Mazda RX4 Wag Hornet 4 Drive Valiant
#> 21.12497 20.41604 19.47080 18.78968
#> Merc 280 Merc 280C Ferrari Dino
#> 18.84528 18.84528 20.70795
#>
#> $`8`
#> Hornet Sportabout Duster 360 Merc 450SE
#> 16.32604 16.04103 14.94481
#> Merc 450SL Merc 450SLC Cadillac Fleetwood
#> 15.69024 15.58061 12.35773
#> Lincoln Continental Chrysler Imperial Dodge Challenger
#> 11.97625 12.14945 16.15065
#> AMC Javelin Camaro Z28 Pontiac Firebird
#> 16.33700 15.44907 15.43811
#> Ford Pantera L Maserati Bora
#> 16.91800 16.04103
#>
# Vectorizing a function over multiple arguments
df <- data.frame(
x = c("apple", "banana", "cherry"),
pattern = c("p", "n", "h"),
replacement = c("P", "N", "H"),
stringsAsFactors = FALSE
)
pmap(df, gsub)
#> [[1]]
#> [1] "aPPle"
#>
#> [[2]]
#> [1] "baNaNa"
#>
#> [[3]]
#> [1] "cHerry"
#>
pmap_chr(df, gsub)
#> [1] "aPPle" "baNaNa" "cHerry"
# Use `...` to absorb unused components of input list .l
df <- data.frame(
x = 1:3,
y = 10:12,
z = letters[1:3]
)
plus <- function(x, y) x + y
if (FALSE) {
# this won't work
pmap(df, plus)
}
# but this will
plus2 <- function(x, y, ...) x + y
pmap_dbl(df, plus2)
#> [1] 11 13 15
# The "p" for "parallel" in pmap() is the same as in base::pmin()
# and base::pmax()
df <- data.frame(
x = c(1, 2, 5),
y = c(5, 4, 8)
)
# all produce the same result
pmin(df$x, df$y)
#> [1] 1 2 5
map2_dbl(df$x, df$y, min)
#> [1] 1 2 5
pmap_dbl(df, min)
#> [1] 1 2 5
# If you want to bind the results of your function rowwise, use:
# map2_dfr() or pmap_dfr()
ex_fun <- function(arg1, arg2){
col <- arg1 + arg2
x <- as.data.frame(col)
}
arg1 <- 1:4
arg2 <- 10:13
map2_dfr(arg1, arg2, ex_fun)
#> col
#> 1 11
#> 2 13
#> 3 15
#> 4 17
# If instead you want to bind by columns, use map2_dfc() or pmap_dfc()
map2_dfc(arg1, arg2, ex_fun)
#> New names:
#> * col -> col...1
#> * col -> col...2
#> * col -> col...3
#> * col -> col...4
#> col...1 col...2 col...3 col...4
#> 1 11 13 15 17